@@ -3,11 +3,13 @@ from numba import prange
 
 from .._common import dist2d, jitted, norm2d
 from .._interp import interp2d
-from ._common import first_index, shrink
+from ._common import shrink
 
 
-@jitted("f8[:, :](f8[:], f8[:], f8[:, :], f8[:, :], f8, f8, f8, f8, f8, b1)")
-def _ray2d(z, x, zgrad, xgrad, zend, xend, zsrc, xsrc, stepsize, honor_grid):
+@jitted(
+    "Tuple((f8[:, :], i4))(f8[:], f8[:], f8[:, :], f8[:, :], f8, f8, f8, f8, f8, i4, b1)"
+)
+def _ray2d(z, x, zgrad, xgrad, zend, xend, zsrc, xsrc, stepsize, max_step, honor_grid):
     """Perform a posteriori 2D ray-tracing."""
     condz = z[0] <= zend <= z[-1]
     condx = x[0] <= xend <= x[-1]
@@ -24,10 +26,15 @@ def _ray2d(z, x, zgrad, xgrad, zend, xend, zsrc, xsrc, stepsize, honor_grid):
         lower = numpy.array([zmin, xmin])
         upper = numpy.array([z[min(i + 1, nz - 1)], x[min(j + 1, nx - 1)]])
 
+        isrc = numpy.searchsorted(z, zsrc, side="right") - 1
+        jsrc = numpy.searchsorted(x, xsrc, side="right") - 1
+
+    count = 1
     pcur = numpy.array([zend, xend], dtype=numpy.float64)
     delta = numpy.empty(2, dtype=numpy.float64)
-    ray = [pcur.copy()]
-    while dist2d(zsrc, xsrc, pcur[0], pcur[1]) > stepsize:
+    ray = numpy.empty((max_step, 2), dtype=numpy.float64)
+    ray[0] = pcur.copy()
+    while dist2d(zsrc, xsrc, pcur[0], pcur[1]) >= stepsize:
         gz = interp2d(z, x, zgrad, pcur)
         gx = interp2d(z, x, xgrad, pcur)
         gn = norm2d(gz, gx)
@@ -52,46 +59,80 @@ def _ray2d(z, x, zgrad, xgrad, zend, xend, zsrc, xsrc, stepsize, honor_grid):
                 upper[0] = z[i + 1]
                 upper[1] = x[j + 1]
 
-                ray.append(pcur.copy())
+                # Handle precision issues due to fac
+                pcur[0] = numpy.round(pcur[0], 8)
+                pcur[1] = numpy.round(pcur[1], 8)
+
+                if (pcur != ray[count - 1]).any():
+                    ray[count] = pcur.copy()
+                    count += 1
+
+                else:
+                    ray[count - 1] = pcur.copy()
+
+                if i == isrc and j == jsrc:
+                    break
 
         else:
             pcur -= delta
-            ray.append(pcur.copy())
+            ray[count] = pcur.copy()
+            count += 1
 
-    ray.append(numpy.array([zsrc, xsrc], dtype=numpy.float64))
-    out = numpy.empty((len(ray), 2), dtype=numpy.float64)
-    for i, r in enumerate(ray):
-        out[-i - 1] = r
+        if count >= max_step:
+            raise RuntimeError("maximum number of steps reached")
 
-    return out
+    ray[count] = numpy.array([zsrc, xsrc], dtype=numpy.float64)
+
+    return ray, count
 
 
 @jitted(parallel=True)
 def _ray2d_vectorized(
-    z, x, zgrad, xgrad, zend, xend, zsrc, xsrc, stepsize, honor_grid=False
+    z, x, zgrad, xgrad, zend, xend, zsrc, xsrc, stepsize, max_step, honor_grid=False
 ):
     """Perform ray-tracing in parallel for different points."""
-    out = []
-    for i in prange(len(zend)):
-        out.append(
-            _ray2d(
-                z, x, zgrad, xgrad, zend[i], xend[i], zsrc, xsrc, stepsize, honor_grid
-            )
+    n = len(zend)
+    rays = numpy.empty((n, max_step, 2), dtype=numpy.float64)
+    counts = numpy.empty(n, dtype=numpy.int32)
+    for i in prange(n):
+        rays[i], counts[i] = _ray2d(
+            z,
+            x,
+            zgrad,
+            xgrad,
+            zend[i],
+            xend[i],
+            zsrc,
+            xsrc,
+            stepsize,
+            max_step,
+            honor_grid,
         )
 
-    return out
+    return rays, counts
 
 
 @jitted
-def ray2d(z, x, zgrad, xgrad, p, src, stepsize, honor_grid=False):
+def ray2d(z, x, zgrad, xgrad, p, src, stepsize, max_step, honor_grid=False):
     """Perform ray-tracing."""
     if p.ndim == 1:
-        return _ray2d(
-            z, x, zgrad, xgrad, p[0], p[1], src[0], src[1], stepsize, honor_grid
+        ray, count = _ray2d(
+            z,
+            x,
+            zgrad,
+            xgrad,
+            p[0],
+            p[1],
+            src[0],
+            src[1],
+            stepsize,
+            max_step,
+            honor_grid,
         )
+        return ray[count::-1]
 
     else:
-        rays = _ray2d_vectorized(
+        rays, counts = _ray2d_vectorized(
             z,
             x,
             zgrad,
@@ -101,11 +142,7 @@ def ray2d(z, x, zgrad, xgrad, p, src, stepsize, honor_grid=False):
             src[0],
             src[1],
             stepsize,
+            max_step,
             honor_grid,
         )
-
-        # Hack: append does not work in parallel, sort back list
-        end_points = [ray[-1] for ray in rays]
-        idx = [first_index(x, end_points) for x in p]
-
-        return [rays[i] for i in idx]
+        return [ray[count::-1] for ray, count in zip(rays, counts)]
