@@ -3,11 +3,11 @@ from numba import prange
 
 from .._common import dist3d, jitted, norm3d
 from .._interp import interp3d
-from ._common import first_index, shrink
+from ._common import shrink
 
 
 @jitted(
-    "f8[:, :](f8[:], f8[:], f8[:], f8[:, :, :], f8[:, :, :], f8[:, :, :], f8, f8, f8, f8, f8, f8, f8, b1)"
+    "Tuple((f8[:, :], i4))(f8[:], f8[:], f8[:], f8[:, :, :], f8[:, :, :], f8[:, :, :], f8, f8, f8, f8, f8, f8, f8, i4, b1)"
 )
 def _ray3d(
     z,
@@ -23,6 +23,7 @@ def _ray3d(
     xsrc,
     ysrc,
     stepsize,
+    max_step,
     honor_grid,
 ):
     """Perform a posteriori 3D ray-tracing."""
@@ -46,10 +47,16 @@ def _ray3d(
             [z[min(i + 1, nz - 1)], x[min(j + 1, nx - 1)], y[min(k + 1, ny - 1)]]
         )
 
+        isrc = numpy.searchsorted(z, zsrc, side="right") - 1
+        jsrc = numpy.searchsorted(x, xsrc, side="right") - 1
+        ksrc = numpy.searchsorted(y, ysrc, side="right") - 1
+
+    count = 1
     pcur = numpy.array([zend, xend, yend], dtype=numpy.float64)
     delta = numpy.empty(3, dtype=numpy.float64)
-    ray = [pcur.copy()]
-    while dist3d(zsrc, xsrc, ysrc, pcur[0], pcur[1], pcur[2]) > stepsize:
+    ray = numpy.empty((max_step, 3), dtype=numpy.float64)
+    ray[0] = pcur.copy()
+    while dist3d(zsrc, xsrc, ysrc, pcur[0], pcur[1], pcur[2]) >= stepsize:
         gz = interp3d(z, x, y, zgrad, pcur)
         gx = interp3d(z, x, y, xgrad, pcur)
         gy = interp3d(z, x, y, ygrad, pcur)
@@ -79,18 +86,32 @@ def _ray3d(
                 upper[1] = x[j + 1]
                 upper[2] = y[k + 1]
 
-                ray.append(pcur.copy())
+                # Handle precision issues due to fac
+                pcur[0] = numpy.round(pcur[0], 8)
+                pcur[1] = numpy.round(pcur[1], 8)
+                pcur[2] = numpy.round(pcur[2], 8)
+
+                if (pcur != ray[count - 1]).any():
+                    ray[count] = pcur.copy()
+                    count += 1
+
+                else:
+                    ray[count - 1] = pcur.copy()
+
+                if i == isrc and j == jsrc and k == ksrc:
+                    break
 
         else:
             pcur -= delta
-            ray.append(pcur.copy())
+            ray[count] = pcur.copy()
+            count += 1
 
-    ray.append(numpy.array([zsrc, xsrc, ysrc], dtype=numpy.float64))
-    out = numpy.empty((len(ray), 3), dtype=numpy.float64)
-    for i, r in enumerate(ray):
-        out[-i - 1] = r
+        if count >= max_step:
+            raise RuntimeError("maximum number of steps reached")
 
-    return out
+    ray[count] = numpy.array([zsrc, xsrc, ysrc], dtype=numpy.float64)
+
+    return ray, count
 
 
 @jitted(parallel=True)
@@ -108,38 +129,40 @@ def _ray3d_vectorized(
     xsrc,
     ysrc,
     stepsize,
+    max_step,
     honor_grid=False,
 ):
     """Perform ray-tracing in parallel for different points."""
-    out = []
-    for i in prange(len(zend)):
-        out.append(
-            _ray3d(
-                z,
-                x,
-                y,
-                zgrad,
-                xgrad,
-                ygrad,
-                zend[i],
-                xend[i],
-                yend[i],
-                zsrc,
-                xsrc,
-                ysrc,
-                stepsize,
-                honor_grid,
-            )
+    n = len(zend)
+    rays = numpy.empty((n, max_step, 3), dtype=numpy.float64)
+    counts = numpy.empty(n, dtype=numpy.int32)
+    for i in prange(n):
+        rays[i], counts[i] = _ray3d(
+            z,
+            x,
+            y,
+            zgrad,
+            xgrad,
+            ygrad,
+            zend[i],
+            xend[i],
+            yend[i],
+            zsrc,
+            xsrc,
+            ysrc,
+            stepsize,
+            max_step,
+            honor_grid,
         )
 
-    return out
+    return rays, counts
 
 
 @jitted
-def ray3d(z, x, y, zgrad, xgrad, ygrad, p, src, stepsize, honor_grid=False):
+def ray3d(z, x, y, zgrad, xgrad, ygrad, p, src, stepsize, max_step, honor_grid=False):
     """Perform ray-tracing."""
     if p.ndim == 1:
-        return _ray3d(
+        ray, count = _ray3d(
             z,
             x,
             y,
@@ -153,11 +176,13 @@ def ray3d(z, x, y, zgrad, xgrad, ygrad, p, src, stepsize, honor_grid=False):
             src[1],
             src[2],
             stepsize,
+            max_step,
             honor_grid,
         )
+        return ray[count::-1]
 
     else:
-        rays = _ray3d_vectorized(
+        rays, counts = _ray3d_vectorized(
             z,
             x,
             y,
@@ -171,11 +196,7 @@ def ray3d(z, x, y, zgrad, xgrad, ygrad, p, src, stepsize, honor_grid=False):
             src[1],
             src[2],
             stepsize,
+            max_step,
             honor_grid,
         )
-
-        # Hack: append does not work in parallel, sort back list
-        end_points = [ray[-1] for ray in rays]
-        idx = [first_index(x, end_points) for x in p]
-
-        return [rays[i] for i in idx]
+        return [ray[count::-1] for ray, count in zip(rays, counts)]
