@@ -11,12 +11,7 @@ from helpers import allclose, eik2d
         ([1.5, 1.5], 2.12132034),
         ([-1.0, -1.0], numpy.nan),
         (
-            [
-                [0.0, 0.0],
-                [15.0, 0.0],
-                [15.0, 15.0],
-                [0.0, 15.0],
-            ],
+            [[0.0, 0.0], [15.0, 0.0], [15.0, 15.0], [0.0, 15.0],],
             [0.0, 15.0, 21.21320343, 15.0],
         ),
     ),
